@@ -1,7 +1,3 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import absolute_import, division, print_function
-
 from typing import Optional, Union
 
 try:
@@ -14,24 +10,26 @@ import warnings
 
 import numpy as np
 import pandas as pd
-import typic
-from tstoolbox import tstoolbox, tsutils
+from pydantic import PositiveInt, confloat, validate_arguments
+from toolbox_utils import tsutils
+from tstoolbox import tstoolbox
 
+from . import tdew as tdew_melo
 from .melodist.melodist.humidity import (
     calculate_month_hour_precip_mean,
     disaggregate_humidity,
 )
-from .melodist.melodist.precipitation import disagg_prec
 from .melodist.melodist.radiation import disaggregate_radiation
 from .melodist.melodist.temperature import disaggregate_temperature, get_shift_by_data
-from .melodist.melodist.util.util import get_sun_times
+from .melodist.melodist.util.util import (
+    calculate_mean_daily_course_by_month,
+    get_sun_times,
+)
 from .melodist.melodist.wind import disaggregate_wind
 
-warnings.filterwarnings("ignore")
-
 
 @tsutils.transform_args(source_units=tsutils.make_list, target_units=tsutils.make_list)
-@typic.al
+@validate_arguments
 def single_target_units(source_units, target_units, default=None, cnt=1):
     if default is None:
         return source_units
@@ -44,9 +42,9 @@ def single_target_units(source_units, target_units, default=None, cnt=1):
         raise ValueError(
             tsutils.error_wrapper(
                 f"""
-Since creating a single disaggregated time-series there can only be
-a single "target_units".  You gave "{target_units}".
-"""
+                Since creating a single disaggregated time-series there can
+                only be a single "target_units".  You gave "{target_units}".
+                """
             )
         )
     if len(source_units) == len(target_units):
@@ -55,17 +53,7 @@ a single "target_units".  You gave "{target_units}".
     return [target_units[0]] * len(source_units)
 
 
-@typic.constrained(ge=-90, le=90)
-class FloatLatitude(float):
-    """-90 <= float <= 90"""
-
-
-@typic.constrained(ge=-180, le=180)
-class FloatLongitude(float):
-    """-180 <= float <= 180"""
-
-
-@typic.al
+@validate_arguments(config=dict(arbitrary_types_allowed=True))
 def temperature(
     method: Literal[
         "sine_min_max", "sine_mean", "sine", "mean_course_min_max", "mean_course_mean"
@@ -85,11 +73,11 @@ def temperature(
     print_input=False,
     target_units=None,
     max_delta: bool = False,
-    temp_min_col: Optional[Union[tsutils.IntGreaterEqualToOne, str, pd.Series]] = None,
-    temp_max_col: Optional[Union[tsutils.IntGreaterEqualToOne, str, pd.Series]] = None,
-    temp_mean_col: Optional[Union[tsutils.IntGreaterEqualToOne, str, pd.Series]] = None,
-    lat: Optional[FloatLatitude] = None,
-    lon: Optional[FloatLongitude] = None,
+    temp_min_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    temp_max_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    temp_mean_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    lat: Optional[confloat(ge=-90, le=90)] = None,
+    lon: Optional[confloat(ge=-180, le=180)] = None,
     hourly: Optional[Union[str, pd.Series]] = None,
 ):
     """Disaggregate daily temperature to hourly temperature."""
@@ -98,22 +86,35 @@ def temperature(
     pd.options.display.width = 60
 
     if (
-        method in ["mean_course_min", "mean_course_mean"] or max_delta is True
+        method in ["mean_course_min_max", "mean_course_mean"]
+        or min_max_time == "sun_loc_shift"
+        or max_delta
     ) and hourly is None:
         raise ValueError(
             tsutils.error_wrapper(
                 """
-The methods "mean_course_min", "mean_course_mean", or if `max_delta` is
-True, require a HOURLY temperature values in the CSV file specified by the
-keyword `hourly`."""
+                The methods "mean_course_min", "mean_course_mean", or if
+                `max_delta` is True, or if `min_max_time` is "sun_loc_shift"
+                require a HOURLY temperature values in the CSV file specified
+                by the keyword `hourly`.
+                """
             )
         )
 
-    if method in ["mean_course_min", "mean_course_mean"] or max_delta is True:
+    if (
+        method in ["mean_course_min_max", "mean_course_mean"]
+        or min_max_time == "sun_loc_shift"
+        or max_delta
+    ):
         hourly = tstoolbox.read(hourly)
+        mean_course = calculate_mean_daily_course_by_month(
+            hourly.squeeze(), normalize=True
+        )
+    else:
+        mean_course = None
 
-    if max_delta is True:
-        max_delta = get_shift_by_data(hourly, lon, lat, round(lon / 15.0))
+    if min_max_time == "sun_loc_shift" or max_delta:
+        max_delta = get_shift_by_data(hourly.squeeze(), lon, lat, round(lon / 15.0))
     else:
         max_delta = None
 
@@ -121,28 +122,25 @@ keyword `hourly`."""
         raise ValueError(
             tsutils.error_wrapper(
                 f"""
-For "temperature" disaggregation you need to supply the daily minimum
-column (name or number, data column numbering starts at 1) and the daily
-maximum column (name or number).
+                For "temperature" disaggregation you need to supply the daily
+                minimum column (name or number, data column numbering starts at
+                1) and the daily maximum column (name or number).
 
-Instead `temp_min_col` is {temp_min_col} and `temp_max_col` is
-{temp_max_col}"""
+                Instead `temp_min_col` is {temp_min_col} and `temp_max_col` is
+                {temp_max_col}
+                """
             )
         )
 
-    columns = []
     try:
         temp_min_col = int(temp_min_col)
     except TypeError:
         pass
-    columns.append(temp_min_col)
-
     try:
         temp_max_col = int(temp_max_col)
     except TypeError:
         pass
-    columns.append(temp_max_col)
-
+    columns = [temp_min_col, temp_max_col]
     if temp_mean_col is not None:
         try:
             temp_mean_col = int(temp_mean_col)
@@ -172,15 +170,15 @@ Instead `temp_min_col` is {temp_min_col} and `temp_max_col` is
     if any((tsd.tmax <= tsd.tmin).dropna()):
         raise ValueError(
             tsutils.error_wrapper(
+                f"""
+                On the following dates:
+
+                {tsd[tsd.tmax <= tsd.tmin].index},
+
+                minimum temperature values in column "{temp_min_col}" are
+                greater than or equal to the maximum temperature values in
+                column "{temp_max_col}".
                 """
-On the following dates:
-
-{},
-
-minimum temperature values in column "{}" are greater than or equal to
-the maximum temperature values in column "{}".""".format(
-                    tsd[tsd.tmax <= tsd.tmin].index, temp_min_col, temp_max_col
-                )
             )
         )
 
@@ -188,8 +186,10 @@ the maximum temperature values in column "{}".""".format(
         warnings.warn(
             tsutils.error_wrapper(
                 """
-Since `temp_mean_col` is None, the average daily temperature will be
-estimated by the average of `temp_min_col` and `temp_max_col`"""
+                Since `temp_mean_col` is None, the average daily temperature
+                will be estimated by the average of `temp_min_col` and
+                `temp_max_col`
+                """
             )
         )
         tsd["temp"] = (tsd.tmin + tsd.tmax) / 2.0
@@ -197,14 +197,15 @@ estimated by the average of `temp_min_col` and `temp_max_col`"""
         if any((tsd.tmin >= tsd.temp).dropna()) or any((tsd.tmax <= tsd.temp).dropna()):
             raise ValueError(
                 tsutils.error_wrapper(
+                    f"""
+                    On the following dates:
+
+                    {tsd[tsd.tmin >= tsd.temp | tsd.tmax <= tsd.temp]},
+
+                    the daily average is either below or equal to the minimum
+                    temperature in column {temp_min_col} or higher or equal to
+                    the maximum temperature in column {temp_max_col}.
                     """
-On the following dates:
-
-{tsd[tsd.tmin >= tsd.temp | tsd.tmax <= tsd.temp]},
-
-the daily average is either below or equal to the minimum temperature in column
-{temp_min_col}
-or higher or equal to the maximum temperature in column {}."""
                 )
             )
 
@@ -217,23 +218,25 @@ or higher or equal to the maximum temperature in column {}."""
         sun_times.sunnoon = 12
         sun_times.sunset = 19
         sun_times.daylength = 12
-    else:
-        if lat is None or lon is None:
-            raise ValueError(
-                tsutils.error_wrapper(
-                    f"""
-The `min_max_time` options other than "fix" require calculation of
-sunrise, sun noon, sunset, and day length.  The calculation requires the
-latitude with keyword "lat" and longitude with keyword "lon".
-You gave:
+    elif lat is None or lon is None:
+        raise ValueError(
+            tsutils.error_wrapper(
+                f"""
+                The `min_max_time` options other than "fix" require calculation
+                of sunrise, sun noon, sunset, and day length.  The calculation
+                requires the latitude with keyword "lat" and longitude with
+                keyword "lon".
 
-    lat={lat}
+                You gave:
 
-    lon={lon}
-"""
-                )
+                lat={lat}
+
+                lon={lon}
+                """
             )
+        )
 
+    else:
         sun_times = get_sun_times(tsd.index, float(lon), float(lat), round(lon / 15.0))
 
     ntsd = pd.DataFrame(
@@ -243,17 +246,270 @@ You gave:
             min_max_time=min_max_time,
             mod_nighttime=mod_nighttime,
             max_delta=max_delta,
-            mean_course=hourly,
+            mean_course=mean_course,
             sun_times=sun_times,
         )
     )
 
-    ntsd.columns = ["temperature:{}:disagg".format(target_units[0])]
+    ntsd.columns = [f"temperature:{target_units[0]}:disagg"]
 
     return tsutils.return_input(print_input, tsd, ntsd)
 
 
-@typic.al
+@validate_arguments(config=dict(arbitrary_types_allowed=True))
+def prepare_hum_tdew(
+    method: Literal[
+        "equal",
+        "minimal",
+        "dewpoint_regression",
+        "linear_dewpoint_variation",
+        "min_max",
+        "month_hour_precip_mean",
+    ],
+    source_units,
+    input_ts="-",
+    columns=None,
+    start_date=None,
+    end_date=None,
+    dropna="no",
+    clean=False,
+    round_index=None,
+    skiprows=None,
+    index_type="datetime",
+    names=None,
+    target_units=None,
+    hum_min_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    hum_max_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    hum_mean_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    temp_min_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    temp_max_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    precip_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    a0=None,
+    a1=None,
+    kr=None,
+    hourly_temp=None,
+    hourly_precip_hum=None,
+    preserve_daily_mean=None,
+):
+    """Disaggregate daily humidity to hourly humidity data."""
+    target_units = single_target_units(source_units, target_units, "")
+
+    if method == "equal" and hum_mean_col is None:
+        raise ValueError(
+            tsutils.error_wrapper(
+                """
+                If `method` is "equal" then the mean daily humidity is
+                a required column identified with the keyword `hum_mean_col`
+                """
+            )
+        )
+
+    if method == "month_hour_precip_mean" and precip_col is None:
+        raise ValueError(
+            tsutils.error_wrapper(
+                """
+                If `method` is "month_hour_precip_mean" then the daily precip
+                is a required column identified with the keyword
+                `precip_col`
+                """
+            )
+        )
+
+    if (
+        method in ["minimal", "dewpoint_regression", "linear_dewpoint_variation"]
+        and temp_min_col is None
+    ):
+        raise ValueError(
+            tsutils.error_wrapper(
+                """
+                If `method` is "minimal", "dewpoint_regression", or
+                "linear_dewpoint_variation" then the minimum daily temperature
+                is a required column identified with the keyword
+                `temp_min_col`.
+                """
+            )
+        )
+
+    if method == "min_max" and (
+        hum_min_col is None
+        or hum_max_col is None
+        or temp_min_col is None
+        or temp_max_col is None
+    ):
+        raise ValueError(
+            tsutils.error_wrapper(
+                f"""
+                If `method` is "min_max" then:
+
+                Minimum daily humidity is a required column identified with the
+                keyword `hum_min_col`.  You gave {hum_min_col}.
+
+                Maximum daily humidity is a required column identified with the
+                keyword `hum_max_col`.  You gave {hum_max_col}.
+
+                Minimum daily temperature is a required column identified with
+                the keyword `temp_min_col`.  You gave {temp_min_col}.
+
+                Maximum daily temperature is a required column identified with
+                the keyword `temp_max_col`.  You gave {temp_max_col}.
+                """
+            )
+        )
+
+    if method in ["dewpoint_regression", "linear_dewpoint_variation"] and (
+        a0 is None or a1 is None
+    ):
+        raise ValueError(
+            tsutils.error_wrapper(
+                """
+                If `method` is "dewpoint_regression" or
+                "linear_dewpoint_variation" then a0 and a1 must be given.
+                """
+            )
+        )
+
+    if method == "linear_dewpoint_variation" and kr is None:
+        raise ValueError(
+            tsutils.error_wrapper(
+                """
+                If `method` is "linear_dewpoint_variation" then kr must be
+                given
+                """
+            )
+        )
+
+    if (
+        method
+        in [
+            "minimal",
+            "dewpoint_regression",
+            "linear_dewpoint_variation",
+            "min_max",
+        ]
+        and hourly_temp is None
+    ):
+        raise ValueError(
+            tsutils.error_wrapper(
+                """
+                If `method` is "minimal", "dewpoint_regression",
+                "linear_dewpoint_variation", or "min_max" then hourly
+                temperature is required identified by the filename in keyword
+                `hourly_temp`.
+                """
+            )
+        )
+
+    pd.options.display.width = 60
+
+    columns = []
+    if method == "equal":
+        try:
+            hum_mean_col = int(hum_mean_col)
+        except TypeError:
+            pass
+        columns.append(hum_mean_col)
+
+    if method == "min_max":
+        try:
+            temp_min_col = int(temp_min_col)
+        except TypeError:
+            pass
+        columns.append(temp_min_col)
+        try:
+            temp_max_col = int(temp_max_col)
+        except TypeError:
+            pass
+        columns.append(temp_max_col)
+        try:
+            hum_min_col = int(hum_min_col)
+        except TypeError:
+            pass
+        columns.append(hum_min_col)
+        try:
+            hum_max_col = int(hum_max_col)
+        except TypeError:
+            pass
+        columns.append(hum_max_col)
+
+    if method in ["minimal", "dewpoint_regression", "linear_dewpoint_variation"]:
+        try:
+            temp_min_col = int(temp_min_col)
+        except TypeError:
+            pass
+        columns.append(temp_min_col)
+
+    if method == "month_hour_precip_mean":
+        try:
+            precip_col = int(precip_col)
+        except TypeError:
+            pass
+        columns.append(precip_col)
+
+    if preserve_daily_mean is not None and method in [
+        "minimal",
+        "dewpoint_regression",
+        "linear_dewpoint_variation",
+        "min_max",
+        "month_hour_precip_mean",
+    ]:
+        try:
+            hum_mean_col = int(preserve_daily_mean)
+        except TypeError:
+            pass
+        columns.append(hum_mean_col)
+
+    tsd = tsutils.common_kwds(
+        tsutils.read_iso_ts(
+            input_ts, skiprows=skiprows, names=names, index_type=index_type
+        ),
+        start_date=start_date,
+        end_date=end_date,
+        pick=columns,
+        round_index=round_index,
+        dropna=dropna,
+        source_units=source_units,
+        target_units=target_units,
+        clean=clean,
+    )
+
+    if method == "equal":
+        tsd.columns = ["hum"]
+
+    if preserve_daily_mean is not None:
+        if method in ["minimal", "dewpoint_regression", "linear_dewpoint_variation"]:
+            tsd.columns = ["tmin", "hum"]
+        if method == "min_max":
+            tsd.columns = ["tmin", "tmax", "hum_min", "hum_max", "hum"]
+        elif method == "month_hour_precip_mean":
+            tsd.columns = ["precip", "hum"]
+        preserve_daily_mean = True
+    else:
+        if method in ["minimal", "dewpoint_regression", "linear_dewpoint_variation"]:
+            tsd.columns = ["tmin"]
+        if method == "min_max":
+            tsd.columns = ["tmin", "tmax", "hum_min", "hum_max"]
+
+        elif method == "month_hour_precip_mean":
+            tsd.columns = ["precip"]
+    if method in [
+        "minimal",
+        "dewpoint_regression",
+        "linear_dewpoint_variation",
+        "min_max",
+    ]:
+        hourly_temp = tstoolbox.read(hourly_temp)
+        hourly_temp = hourly_temp.astype(float).squeeze()
+
+    if method == "month_hour_precip_mean":
+        hourly_precip_hum = tstoolbox.read(hourly_precip_hum)
+        month_hour_precip_mean = calculate_month_hour_precip_mean(hourly_precip_hum)
+    else:
+        month_hour_precip_mean = "None"
+
+    return tsd, hourly_temp, month_hour_precip_mean
+
+
+@validate_arguments(config=dict(arbitrary_types_allowed=True))
 def humidity(
     method: Literal[
         "equal",
@@ -276,154 +532,54 @@ def humidity(
     names=None,
     target_units=None,
     print_input=False,
-    hum_min_col: Optional[Union[tsutils.IntGreaterEqualToOne, str, pd.Series]] = None,
-    hum_max_col: Optional[Union[tsutils.IntGreaterEqualToOne, str, pd.Series]] = None,
-    hum_mean_col: Optional[Union[tsutils.IntGreaterEqualToOne, str, pd.Series]] = None,
-    temp_min_col: Optional[Union[tsutils.IntGreaterEqualToOne, str, pd.Series]] = None,
-    temp_max_col: Optional[Union[tsutils.IntGreaterEqualToOne, str, pd.Series]] = None,
+    hum_min_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    hum_max_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    hum_mean_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    temp_min_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    temp_max_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    precip_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
     a0=None,
     a1=None,
     kr=None,
     hourly_temp=None,
+    hourly_precip_hum=None,
     preserve_daily_mean=None,
 ):
     """Disaggregate daily humidity to hourly humidity data."""
     target_units = single_target_units(source_units, target_units, "")
 
-    if method == "equal" and hum_mean_col is None:
-        raise ValueError(
-            tsutils.error_wrapper(
-                """
-If `method` is "equal" then the mean daily humidity is a required column
-identified with the keyword `hum_mean_col`"""
-            )
-        )
-
-    if (
-        method in ["minimal", "dewpoint_regression", "linear_dewpoint_variation"]
-        and temp_min_col is None
-    ):
-        raise ValueError(
-            tsutils.error_wrapper(
-                """
-If `method` is "minimal", "dewpoint_regression", or
-"linear_dewpoint_variation" then the minimum daily temperature is a required
-column identified with the keyword `temp_min_col`."""
-            )
-        )
-
-    if method == "min_max" and (
-        hum_min_col is None
-        or hum_max_col is None
-        or temp_min_col is None
-        or temp_max_col is None
-    ):
-        raise ValueError(
-            tsutils.error_wrapper(
-                f"""
-If `method` is "min_max" then:
-
-Minimum daily humidity is a required column identified with the keyword
-`hum_min_col`.  You gave {hum_min_col}.
-
-Maximum daily humidity is a required column identified with the keyword
-`hum_max_col`.  You gave {hum_max_col}.
-
-Minimum daily temperature is a required column identified with the
-keyword `temp_min_col`.  You gave {temp_min_col}.
-
-Maximum daily temperature is a required column identified with the
-keyword `temp_max_col`.  You gave {temp_max_col}.
-"""
-            )
-        )
-
-    if method in ["dewpoint_regression", "linear_dewpoint_variation"] and (
-        a0 is None or a1 is None
-    ):
-        raise ValueError(
-            tsutils.error_wrapper(
-                """
-If `method` is "dewpoint_regression" or "linear_dewpoint_variation" then
-a0 and a1 must be given."""
-            )
-        )
-
-    if method == "linear_dewpoint_variation" and kr is None:
-        raise ValueError(
-            tsutils.error_wrapper(
-                """
-If `method` is "linear_dewpoint_variation" then kr must be given"""
-            )
-        )
-
-    if (
-        method
-        in [
-            "minimal",
-            "dewpoint_regression",
-            "linear_dewpoint_variation",
-            "min_max",
-            "month_hour_precip_mean",
-        ]
-        and hourly_temp is None
-    ):
-        raise ValueError(
-            tsutils.error_wrapper(
-                """
-If `method` is "minimal", "dewpoint_regression",
-"linear_dewpoint_variation", or "min_max" then hourly temperature is
-required identified by the filename in keyword `hourly_temp`."""
-            )
-        )
-
-    pd.options.display.width = 60
-
-    tsd = tsutils.common_kwds(
-        tsutils.read_iso_ts(
-            input_ts, skiprows=skiprows, names=names, index_type=index_type
-        ),
+    tsd, hourly_temp, month_hour_precip_mean = prepare_hum_tdew(
+        method,
+        source_units,
+        input_ts=input_ts,
+        columns=columns,
         start_date=start_date,
         end_date=end_date,
-        pick=columns,
-        round_index=round_index,
         dropna=dropna,
-        source_units=source_units,
-        target_units=target_units,
         clean=clean,
+        round_index=round_index,
+        skiprows=skiprows,
+        index_type=index_type,
+        names=names,
+        target_units=target_units,
+        print_input=print_input,
+        hum_min_col=hum_min_col,
+        hum_max_col=hum_max_col,
+        hum_mean_col=hum_mean_col,
+        temp_min_col=temp_min_col,
+        temp_max_col=temp_max_col,
+        precip_col=precip_col,
+        a0=a0,
+        a1=a1,
+        kr=kr,
+        hourly_temp=hourly_temp,
+        hourly_precip_hum=hourly_precip_hum,
+        preserve_daily_mean=preserve_daily_mean,
     )
 
-    if method == "equal":
-        tsd["hum"] = tsd["hum_mean_col"]
-
-    if method in ["minimal", "dewpoint_regression", "linear_dewpoint_variation"]:
-        tsd["tmin"] = tsd["temp_min_col"]
-
-    if method == "min_max":
-        tsd["hum_min"] = tsd["hum_min_col"]
-        tsd["hum_max"] = tsd["hum_max_col"]
-        tsd["tmin"] = tsd["temp_min_col"]
-        tsd["tmax"] = tsd["temp_max_col"]
-
-    if preserve_daily_mean is not None:
-        tsd["hum"] = tsd[preserve_daily_mean]
-        preserve_daily_mean = True
-
-    if method in [
-        "minimal",
-        "dewpoint_regression",
-        "linear_dewpoint_variation",
-        "min_max",
-        "month_hour_precip_mean",
-    ]:
-        hourly_temp = tstoolbox.read(hourly_temp)
-
-    if method == "month_hour_precip_mean":
-        month_hour_precip_mean = calculate_month_hour_precip_mean(hourly_temp)
-
     ntsd = pd.DataFrame(
         disaggregate_humidity(
-            tsd,
+            tsd.astype(float),
             method=method,
             temp=hourly_temp,
             a0=a0,
@@ -439,7 +595,96 @@ required identified by the filename in keyword `hourly_temp`."""
     return tsutils.return_input(print_input, tsd, ntsd)
 
 
-@typic.al
+@validate_arguments(config=dict(arbitrary_types_allowed=True))
+def dewpoint_temperature(
+    method: Literal[
+        "equal",
+        "minimal",
+        "dewpoint_regression",
+        "linear_dewpoint_variation",
+        "min_max",
+        "month_hour_precip_mean",
+    ],
+    source_units,
+    input_ts="-",
+    columns=None,
+    start_date=None,
+    end_date=None,
+    dropna="no",
+    clean=False,
+    round_index=None,
+    skiprows=None,
+    index_type="datetime",
+    names=None,
+    target_units=None,
+    print_input=False,
+    hum_min_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    hum_max_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    hum_mean_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    temp_min_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    temp_max_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    precip_col: Optional[Union[PositiveInt, str, pd.Series]] = None,
+    a0=None,
+    a1=None,
+    kr=None,
+    hourly_temp=None,
+    hourly_precip_hum=None,
+    preserve_daily_mean=None,
+):
+    """Disaggregate daily humidity to hourly humidity data."""
+    # target_units = single_target_units(source_units, target_units, "")
+    target_units = single_target_units(source_units, target_units, "degK")
+
+    tsd, hourly_temp, month_hour_precip_mean = prepare_hum_tdew(
+        method,
+        source_units,
+        input_ts=input_ts,
+        columns=columns,
+        start_date=start_date,
+        end_date=end_date,
+        dropna=dropna,
+        clean=clean,
+        round_index=round_index,
+        skiprows=skiprows,
+        index_type=index_type,
+        names=names,
+        target_units=target_units,
+        print_input=print_input,
+        hum_min_col=hum_min_col,
+        hum_max_col=hum_max_col,
+        hum_mean_col=hum_mean_col,
+        temp_min_col=temp_min_col,
+        temp_max_col=temp_max_col,
+        precip_col=precip_col,
+        a0=a0,
+        a1=a1,
+        kr=kr,
+        hourly_temp=hourly_temp,
+        hourly_precip_hum=hourly_precip_hum,
+        preserve_daily_mean=preserve_daily_mean,
+    )
+
+    ntsd = pd.DataFrame(
+        tdew_melo.disaggregate_tdew(
+            tsd.astype(float),
+            method=method,
+            temp=hourly_temp,
+            a0=a0,
+            a1=a1,
+            kr=kr,
+            preserve_daily_mean=preserve_daily_mean,
+            month_hour_precip_mean=month_hour_precip_mean,
+        )
+    )
+
+    ntsd.columns = ["dewpoint_temp:degK:disagg"]
+    ntsd_units = tsutils._normalize_units(
+        ntsd, source_units="degK", target_units=target_units[0]
+    )
+    return tsutils.return_input(print_input, tsd, ntsd)
+
+
+@validate_arguments(config=dict(arbitrary_types_allowed=True))
 def wind_speed(
     method: Literal["equal", "cosine", "random"],
     source_units,
@@ -462,23 +707,30 @@ def wind_speed(
     """Disaggregate daily to hourly data."""
     target_units = single_target_units(source_units, target_units, "m/s")
 
-    target_units = target_units[0] * len(source_units)
-
-    pd.options.display.width = 60
-
     if method == "cosine" and (a is None or b is None or t_shift is None):
         raise ValueError(
             tsutils.error_wrapper(
                 f"""
-For the "cosine" method, requires the `a`, `b`, and `t_shift`
-keywords.  You gave:
+                For the "cosine" method, requires the `a`, `b`, and `t_shift`
+                keywords.  You gave:
 
-a = {a}
+                a = {a}
 
-b = {b}
+                b = {b}
 
-t_shift = {t_shift}
-"""
+                t_shift = {t_shift}
+                """
+            )
+        )
+    if method in ["equal", "random"] and not (
+        a is None or b is None or t_shift is None
+    ):
+        warnings.warn(
+            tsutils.error_wrapper(
+                """
+                The a, b, and t_shift options are ignored for the "equal" and
+                "random" methods.
+                """
             )
         )
     tsd = tsutils.common_kwds(
@@ -495,14 +747,19 @@ t_shift = {t_shift}
         clean=clean,
     )
 
+    ndf = pd.DataFrame()
+    for column_name, column_data in tsd.iteritems():
+        df = disaggregate_wind(column_data, method=method, a=a, b=b, t_shift=t_shift)
+        ndf = ndf.join(df, how="outer")
+
     return tsutils.return_input(
         print_input,
         tsd,
-        pd.DataFrame(disaggregate_wind(tsd, method=method, a=a, b=b, t_shift=t_shift)),
+        ndf,
     )
 
 
-@typic.al
+@validate_arguments(config=dict(arbitrary_types_allowed=True))
 def radiation(
     method: Literal["pot_rad", "pot_rad_via_ssd", "pot_rad_via_bc", "mean_course"],
     source_units,
@@ -539,8 +796,8 @@ def radiation(
         raise ValueError(
             tsutils.error_wrapper(
                 """
-If method is "mean_course" need to supply CSV filename of hourly
-radiation by the `hourly_rad` keyword."""
+                If method is "mean_course" need to supply CSV filename of
+                hourly radiation by the `hourly_rad` keyword."""
             )
         )
 
@@ -548,9 +805,10 @@ radiation by the `hourly_rad` keyword."""
         raise ValueError(
             tsutils.error_wrapper(
                 """
-If method is "pot_rad" or "mean_course" need to supply the daily global
-short wave radiation as column name or index with keyword
-`glob_swr_col`"""
+                If method is "pot_rad" or "mean_course" need to supply the
+                daily global short wave radiation as column name or index with
+                keyword `glob_swr_col`
+                """
             )
         )
 
@@ -558,8 +816,9 @@ short wave radiation as column name or index with keyword
         raise ValueError(
             tsutils.error_wrapper(
                 """
-If method is "pot_rad_via_bc" need to supply the keywords `bristcamp_a`
-and `bristcamp_c`."""
+                If method is "pot_rad_via_bc" need to supply the keywords
+                `bristcamp_a` and `bristcamp_c`.
+                """
             )
         )
 
@@ -607,7 +866,7 @@ and `bristcamp_c`."""
     )
 
 
-@typic.al
+@validate_arguments(config=dict(arbitrary_types_allowed=True))
 def precipitation(
     method: Literal["equal", "cascade", "masterstation"],
     source_units,
@@ -623,7 +882,7 @@ def precipitation(
     target_units=None,
     print_input=False,
     columns=None,
-    masterstation_hour_col: Optional[Union[tsutils.IntGreaterEqualToOne, str]] = None,
+    masterstation_hour_col: Optional[Union[PositiveInt, str]] = None,
 ):
     """Disaggregate daily to hourly data."""
     target_units = single_target_units(source_units, target_units, "mm")
@@ -631,17 +890,18 @@ def precipitation(
     pd.options.display.width = 60
 
     tsd = tsutils.common_kwds(
-        tsutils.read_iso_ts(
-            input_ts, skiprows=skiprows, names=names, index_type=index_type
-        ),
+        input_tsd=tsutils.make_list(input_ts),
+        skiprows=skiprows,
+        index_type=index_type,
         start_date=start_date,
         end_date=end_date,
-        pick=columns,
         round_index=round_index,
+        names=names,
         dropna=dropna,
+        clean=clean,
         source_units=source_units,
         target_units=target_units,
-        clean=clean,
+        usecols=columns,
     )
 
     if method == "masterstation":
@@ -652,10 +912,25 @@ def precipitation(
             # If masterstations_hour_col is a column number:
             masterstation_hour_col = int(masterstation_hour_col) - 1
 
-        masterstation_hour_col = tsd.columns[masterstation_hour_col]
+        try:
+            mhour = tsd[masterstation_hour_col].to_frame()
+        except:
+            mhour = tsutils.common_kwds(
+                input_tsd=tsutils.make_list(input_ts),
+                skiprows=skiprows,
+                index_type=index_type,
+                start_date=start_date,
+                end_date=end_date,
+                round_index=round_index,
+                names=names,
+                dropna=dropna,
+                clean=clean,
+                source_units=source_units,
+                target_units=target_units,
+                usecols=columns,
+            )
 
         # Should only be one hourly column in the input.
-        mhour = tsd[masterstation_hour_col].to_frame()
         dsum = mhour.groupby(pd.Grouper(freq="D")).sum().asfreq("H", method="ffill")
         master = mhour.join(dsum, rsuffix="sum")
         mask = master.iloc[:, 0] > 0.0
@@ -684,7 +959,7 @@ def precipitation(
     return tsutils.return_input(print_input, tsd, ntsd)
 
 
-@typic.al
+@validate_arguments(config=dict(arbitrary_types_allowed=True))
 def evaporation(
     method: Literal["trap", "fixed"],
     source_units,
@@ -700,7 +975,7 @@ def evaporation(
     names=None,
     target_units=None,
     print_input=False,
-    lat: Optional[FloatLatitude] = None,
+    lat: Optional[confloat(ge=-90, le=90)] = None,
 ):
     """Disaggregate daily to hourly data."""
     target_units = single_target_units(source_units, target_units)
@@ -711,8 +986,9 @@ def evaporation(
         raise ValueError(
             tsutils.error_wrapper(
                 f"""
-The "trap" method requires latitude with the `lat` keyword.  You gave
-"{lat}". """
+                The "trap" method requires latitude with the `lat` keyword.
+                You gave "{lat}".
+                """
             )
         )
     tsd = tsutils.common_kwds(
